@@ -74,7 +74,8 @@ from astropy.io import fits
 from astropy.wcs import WCS as awcs
 from cadcutils import exceptions, net
 from caom2utils import FitsParser, WcsParser, main_app, update_blueprint
-from caom2utils import ObsBlueprint, GenericParser
+from caom2utils import ObsBlueprint, GenericParser, gen_proc
+from caom2utils import get_gen_proc_arg_parser
 from caom2utils.legacy import load_config
 from caom2utils.fits2caom2 import _visit, _load_plugin, _update_artifact_meta
 
@@ -93,6 +94,7 @@ from lxml import etree
 from mock import Mock, patch
 from six import StringIO, BytesIO
 
+import importlib
 import os
 import sys
 
@@ -114,10 +116,6 @@ test_plugin_module = os.path.join(TESTDATA_DIR, 'test_plugin.py')
 test_class_plugin_module = os.path.join(TESTDATA_DIR, 'test_plugin_class.py')
 non_conformant_plugin_module = os.path.join(TESTDATA_DIR, 'nonconformant.py')
 
-# to execute only one test in the file set this var to True and comment
-# out the skipif decorator of the test
-single_test = False
-
 
 class MyExitError(Exception):
     pass
@@ -147,7 +145,6 @@ EXPECTED_ENERGY_XML = \
 '''
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_augment_energy():
     bp = ObsBlueprint(energy_axis=1)
     test_fitsparser = FitsParser(sample_file_4axes, bp)
@@ -161,7 +158,6 @@ def test_augment_energy():
     assert result is None, repr(energy)
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_augment_artifact_energy_from_blueprint():
     test_blueprint = ObsBlueprint(energy_axis=1)
     test_blueprint.set('Chunk.energyAxis', 1)
@@ -204,7 +200,6 @@ EXPECTED_POLARIZATION_XML = \
 '''
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_augment_polarization():
     test_fitsparser = FitsParser(sample_file_4axes,
                                  ObsBlueprint(polarization_axis=1))
@@ -219,7 +214,6 @@ def test_augment_polarization():
     assert result is None, result
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_augment_artifact_polarization_from_blueprint():
     test_blueprint = ObsBlueprint(polarization_axis=1)
     test_blueprint.set('Chunk.polarizationAxis', '1')
@@ -277,7 +271,6 @@ EXPECTED_POSITION_XML = \
 '''
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_augment_artifact():
     test_blueprint = ObsBlueprint(position_axes=(1, 2))
     test_fitsparser = FitsParser(sample_file_4axes, test_blueprint)
@@ -296,7 +289,6 @@ def test_augment_artifact():
     assert result is None
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_augment_artifact_position_from_blueprint():
     test_blueprint = ObsBlueprint(position_axes=(1, 2))
     test_blueprint.set('Chunk.positionAxis1', '1')
@@ -356,7 +348,6 @@ EXPECTED_CFHT_WIRCAM_RAW_GUIDE_CUBE_TIME = \
 '''
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_augment_artifact_time():
     test_fitsparser = FitsParser(sample_file_time_axes,
                                  ObsBlueprint(time_axis=1))
@@ -375,7 +366,6 @@ def test_augment_artifact_time():
     assert result is None
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_augment_artifact_time_from_blueprint():
     test_blueprint = ObsBlueprint(time_axis=1)
     test_blueprint.set('Chunk.timeAxis', '1')
@@ -401,7 +391,6 @@ def test_augment_artifact_time_from_blueprint():
     assert result is None
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_get_wcs_values():
     w = get_test_wcs(sample_file_4axes)
     test_parser = WcsParser(get_test_header(sample_file_4axes)[0].header,
@@ -441,7 +430,6 @@ def _get_from_str_xml(string_xml, get_func, element_tag):
     return act_obj
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 @patch('sys.exit', Mock(side_effect=[MyExitError, MyExitError, MyExitError,
                                      MyExitError, MyExitError,
                                      MyExitError]))
@@ -581,7 +569,6 @@ caom2:id="d2893703-b21e-425f-b7d0-ca1f58fdc011">
 """
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_augment_observation():
     test_obs_blueprint = ObsBlueprint(position_axes=(1, 2))
     test_obs_blueprint.set('Observation.target.name', 'CGPS Mosaic MA1')
@@ -629,7 +616,6 @@ def test_augment_observation():
     assert diff_result is None
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_get_from_list():
     test_fitsparser = FitsParser(sample_file_4axes)
     test_fitsparser.blueprint = ObsBlueprint()
@@ -638,7 +624,6 @@ def test_get_from_list():
     assert result == ObservationIntentType.SCIENCE
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_update_fits_headers():
     # The rules for the values:
     # all upper case - a FITS keyword
@@ -808,14 +793,12 @@ TEST_OVERRIDES = \
      }}
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_load_config_overrides():
     # cool override file content
     result = load_config(override_file)
     assert result == TEST_OVERRIDES
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_chunk_naxis():
     hdr1 = fits.Header()
     test_blueprint = ObsBlueprint()
@@ -916,7 +899,6 @@ EXPECTED_FILE_SCHEME_XML = """<?xml version='1.0' encoding='UTF-8'?>
 """
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_file_scheme_uris():
     """ Tests that local files as URIs will be accepted and processed."""
 
@@ -974,7 +956,6 @@ EXPECTED_GENERIC_PARSER_FILE_SCHEME_XML = """<?xml version='1.0' encoding='UTF-8
 """
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_generic_parser():
     """ Tests that GenericParser will be created."""
 
@@ -993,7 +974,6 @@ def test_generic_parser():
             assert result is None
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_visit():
     test_obs = _get_obs(EXPECTED_FILE_SCHEME_XML)
 
@@ -1117,7 +1097,6 @@ EXPECTED_POS_RANGE_BOUNDS_XML = '''<caom2:import xmlns:caom2="http://www.opencad
 '''
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_augment_artifact_bounds_range_from_blueprint():
     test_blueprint = ObsBlueprint(energy_axis=1, time_axis=2,
                                   polarization_axis=3,
@@ -1197,7 +1176,6 @@ def test_augment_artifact_bounds_range_from_blueprint():
     assert result is None
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_visit_generic_parser():
     try:
         sys.argv = ['fits2caom2', '--local', 'fname', '--observation',
@@ -1215,15 +1193,14 @@ def test_visit_generic_parser():
         assert False, 'should not get here {}'.format(e)
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 @patch('caom2utils.fits2caom2.Client')
 def test_get_vos_headers(vos_mock):
     test_uri = 'vos://cadc.nrc.ca!vospace/CAOMworkshop/Examples/DAO/' \
                'dao_c122_2016_012725.fits'
-    get_orig = caom2utils.fits2caom2._get_headers_from_fits
+    get_orig = caom2utils.fits2caom2.get_cadc_headers
 
     try:
-        caom2utils.fits2caom2._get_headers_from_fits = Mock(
+        caom2utils.fits2caom2.get_cadc_headers = Mock(
             side_effect=_get_headers)
         test_headers = caom2utils.get_vos_headers(test_uri, subject=None)
         assert test_headers is not None, 'expect result'
@@ -1231,10 +1208,9 @@ def test_get_vos_headers(vos_mock):
         assert test_headers[0]['SIMPLE'] is True, 'SIMPLE header not found'
         assert vos_mock.called, 'mock not called'
     finally:
-        caom2utils.fits2caom2._get_headers_from_fits = get_orig
+        caom2utils.fits2caom2.get_cadc_headers = get_orig
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 @patch('caom2utils.fits2caom2.Client')
 def test_get_vos_meta(vos_mock):
     get_orig = caom2utils.get_vos_headers
@@ -1260,7 +1236,6 @@ def test_get_vos_meta(vos_mock):
         caom2utils.get_vos_headers = get_orig
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_generic_parser1():
     test_key = 'Plane.metaRelease'
     test_value = '2013-10-10'
@@ -1275,7 +1250,6 @@ def test_generic_parser1():
         'original value over-ridden'
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
 def test_get_external_headers():
     test_uri = 'http://localhost/obs23/collection/obsid-1'
     with patch('requests.Session.get') as session_get_mock:
@@ -1289,7 +1263,32 @@ def test_get_external_headers():
         assert session_get_mock.is_called_with(test_uri)
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
+@patch('caom2utils.fits2caom2.get_external_headers')
+def test_get_external_headers_fails(get_external_mock):
+    get_external_mock.return_value = None
+    test_collection = 'TEST_COLLECTION'
+    test_obs_id = 'TEST_OBS_ID'
+    test_uri = 'gemini:{}/abc.fits'.format(test_collection)
+    test_product_id = 'TEST_PRODUCT_ID'
+    test_blueprint = caom2utils.fits2caom2.ObsBlueprint()
+    test_observation = SimpleObservation(collection=test_collection,
+                                         observation_id=test_obs_id,
+                                         algorithm=Algorithm(name='exposure'))
+    test_result = caom2utils.fits2caom2._augment(
+        obs=test_observation,
+        product_id=test_product_id,
+        uri=test_uri,
+        blueprint=test_blueprint,
+        subject=net.Subject(),
+        external_url='https://localhost/files/test_file.fits.gz',
+    )
+    assert test_result is not None, 'expect a result'
+    assert len(test_result.planes.values()) == 1, 'plane added to result'
+    test_plane = test_result.planes[test_product_id]
+    assert len(test_plane.artifacts.values()) == 1, 'artifact added to plane'
+    assert test_uri in test_plane.artifacts.keys(), 'wrong artifact uri'
+
+
 def test_apply_blueprint():
     # test a Gemini case where there are two keywords, one each for
     # different instruments, and the default ends up getting set when the
@@ -1328,7 +1327,29 @@ def test_apply_blueprint():
         test_parser._headers[0]['IMAGESWV'], 'should not be set'
 
 
-@pytest.mark.skipif(single_test, reason='Single test mode')
+def test_apply_blueprint_execute_external():
+    test_module = importlib.import_module(__name__)
+    test_generic_blueprint = ObsBlueprint(module=test_module)
+    test_generic_blueprint.set('Observation.type', '_get_test_obs_type(parameters)')
+
+    # generic parser
+    test_generic_parser = GenericParser(test_generic_blueprint)
+    assert test_generic_parser is not None, \
+        'expect generic construction to complete'
+    assert test_generic_parser._get_from_list('Observation.type', index=0) \
+           == 'generic_parser_value', 'wrong generic plan value'
+
+    # fits parser
+    test_fits_blueprint = ObsBlueprint(module=test_module)
+    test_fits_blueprint.set('Observation.type', '_get_test_obs_type(parameters)')
+    test_fits_parser = FitsParser(src=sample_file_4axes,
+                                  obs_blueprint=test_fits_blueprint)
+    assert test_fits_parser is not None, \
+        'expect fits construction to complete'
+    assert test_fits_parser._get_from_list('Observation.type', index=0) \
+           == 'fits_parser_value', 'wrong fits plan value'
+
+
 def test_update_artifact_meta_errors():
     test_uri = 'gemini:GEMINI/abc.jpg'
     test_artifact = Artifact(uri=test_uri,
@@ -1370,7 +1391,29 @@ def test_update_artifact_meta_errors():
         assert test_artifact.content_checksum is None, 'checksum'
 
 
-def _get_headers(subject):
+@patch('sys.stdout', new_callable=BytesIO)
+@patch('caom2utils.fits2caom2._augment')
+def test_gen_proc_failure(augment_mock, stdout_mock):
+    """ Tests that gen_proc can return -1."""
+
+    augment_mock.return_value = None  # return a broken Observation instance
+    fname = 'file://{}'.format(text_file)
+    sys.argv = ['fits2caom2', '--local', fname,
+                '--observation', 'test_collection_id',
+                'test_observation_id', '--lineage',
+                'test_product_id/ad:TEST/{}'.format(fname)]
+    test_args = get_gen_proc_arg_parser().parse_args()
+    test_blueprints = {'test_collection_id': ObsBlueprint()}
+    test_result = gen_proc(test_args, test_blueprints)
+    assert test_result == -1, 'expect failure'
+    if stdout_mock.getvalue():
+        expected = _get_obs(EXPECTED_GENERIC_PARSER_FILE_SCHEME_XML)
+        actual = _get_obs(stdout_mock.getvalue().decode('ascii'))
+        result = get_differences(expected, actual, 'Observation')
+        assert result is None
+
+
+def _get_headers(file_name, subject):
     x = """SIMPLE  =                    T / Written by IDL:  Fri Oct  6 01:48:35 2017
 BITPIX  =                  -32 / Bits per pixel
 NAXIS   =                    2 / Number of dimensions
@@ -1422,3 +1465,11 @@ def _get_node(uri, limit, force):
     node.props = {'MD5': '5b00b00d4b06aba986c3663d09aa581f',
                   'length': 682560}
     return node
+
+
+def _get_test_obs_type(parameters):
+    assert parameters is not None
+    if parameters.get('header') is None:
+        return 'generic_parser_value'
+    else:
+        return 'fits_parser_value'
